import { Component } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Observable, catchError, of, retry } from 'rxjs';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Employee } from '../Employee';

import { MatCardModule } from '@angular/material/card';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatSnackBar, MatSnackBarModule } from '@angular/material/snack-bar';
import { MatDividerModule } from '@angular/material/divider';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatMenuModule } from '@angular/material/menu';
import { MatTableModule } from '@angular/material/table';
import { MatSortModule } from '@angular/material/sort';

@Component({
  selector: 'app-employee-list',
  standalone: true,
  imports: [
    CommonModule,
    MatCardModule,
    MatButtonModule,
    MatIconModule,
    MatProgressSpinnerModule,
    MatSnackBarModule,
    MatDividerModule,
    MatTooltipModule,
    MatMenuModule,
    MatTableModule,
    MatSortModule
  ],
  templateUrl: './employee-list.component.html',
  host: {
    class: 'block w-full p-6'
  },
  styleUrl: './employee-list.component.css'
})
export class EmployeeListComponent {
  private static readonly EMPLOYEES_ENDPOINT = 'http://localhost:8089/employees';
  private static readonly MAX_RETRIES = 3;
  public employees$: Observable<Employee[]>;
  public readonly displayedColumns: string[] = ['name', 'actions'];

  constructor(
    private readonly httpClient: HttpClient,
    private readonly snackBar: MatSnackBar
  ) {
    this.employees$ = this.fetchEmployees();
  }

  private fetchEmployees(): Observable<Employee[]> {
    return this.httpClient.get<Employee[]>(
      EmployeeListComponent.EMPLOYEES_ENDPOINT,
    ).pipe(
      retry(EmployeeListComponent.MAX_RETRIES),
      catchError((error: HttpErrorResponse) => {
        console.error('Error fetching employees:', error);
        this.showErrorMessage('Failed to load employees. Please try again.');
        return of([]);
      })
    );
  }

  private showErrorMessage(message: string): void {
    this.snackBar.open(message, 'Close', {
      duration: 5000,
      horizontalPosition: 'end',
      verticalPosition: 'bottom',
      panelClass: ['!bg-red-50', '!text-red-900', '!border', '!border-red-100']
    });
  }
}
